// Drill code
// Diyouware TwinTeeth mini-PCB factory,
// Copyright 2015 by V.Aylln.
//
/**
/* write_drill_header() - writes the gcode header
**/
void write_drill_header(string fname, int bit_num) {
	output(fname,"wt") {
		printf(";%s\n",board_name);
		if (current_layer == TOP)
			printf(";Top layer - drill #%d size %f mm\n",bit_num, DRILL_SIZE[bit_num]);
		else
			printf(";Bottom layer, drill #%d size %f mm\n",bit_num, DRILL_SIZE[bit_num]);
		printf(
			";\n"
			//"G28 ; home axis\n"
			"G90 ; use absolute coordinates\n"
			"G21 ; set units to millimeters\n"
			"G0 Z%0.3f F%d\n", DRILL_Z_HIGH , DRILL_FEED_RATE
			);
	}
}
/**
/* write_drill_item() - writes a dispense coordinates item
**/
void write_drill_item(string fname, int i) {

	output(fname,"at") {
		//printf("Drill: %.2f (%.2f %.2f)\n",drill_size_mm[i], scale_x(x[i]), scale_y(y[i]));
		printf("G0 X%.2f Y%.2f Z%.2f F%d\n", scale_x(x[i]), scale_y(y[i]), DRILL_Z_UP, DRILL_FEED_RATE); // move to new position, above board
		printf("G1 Z%.2f F%d\n", DRILL_Z_DOWN, DRILL_FEED_RATE_Z);  // ready to drill.
		printf("G1 Z%.2f F%d\n", DRILL_Z_DOWN + (abs(DRILL_DEPTH)*-1), DRILL_FEED_RATE_Z);  // Drill to depht
		if (DRILL_DWELL > 0)
			printf("G4 P%.1f\n",DRILL_DWELL);  // Wait given milliseconds
		printf("G1 Z%.2f F%d\n", DRILL_Z_UP, DRILL_FEED_RATE_Z);  // High up drilling tool
	}
}
/**
/* write_drill_trailer() - writes the end of the gcode file
**/
void write_drill_trailer(string fname){
	output(fname,"at") {
	printf("G0 Z%0.3f F%d\n", DRILL_Z_HIGH , DRILL_FEED_RATE);
	printf("M84 ; shut down motors, done\n");
	}
}
/**
/* write_drills() - Write all drills of this gcode file
**/
void write_drill_body(string fname, int bit_num) {
	// write items to file
	for (int i = 0; i < drill_index; i++) {
		if (drill_size_mm[index[i]] == DRILL_SIZE[bit_num])
			write_drill_item(fname, index[i]);
	}
}
/**
 * get_drill() - Returns a drill in mm based on the min and max values of the hole.
**/
real get_drill(int hole_size)
{
	real drill_mm;

	drill_mm = u2mm(hole_size);

	for (int i = 0; i < MAX_DRILL_BITS; i++){

		// Check if in range of any drill bit
		if (DRILL_BIT[i] == YES){
			if (drill_mm >= MIN_DRILL_SIZE[i] && drill_mm <= MAX_DRILL_SIZE[i]){
				bit_active[i] = YES;
				return DRILL_SIZE[i];
			}
		}
	}

	show_error("No drill bit defined for ", real_to_string(drill_mm) + "mm\n" + "Please try again.");
	return ERROR;

}

//
// set_hole() - Set a hole drilling
//
void set_hole(int hole_size, int cx, int cy)
{
	drill_size_mm[drill_index] = get_drill(hole_size);

	if (drill_size_mm[drill_index] == ERROR)
		exit(-1);
	x[drill_index] = cx;
	y[drill_index] = cy;

	drill_index++;
}

/**
/* drill_layer() - Generate the files of current layer
**/
void drill_layer(string fname){
	//
	// Generate a file per each active drill bit
	// per layer
	//
	for (int i = 0; i < MAX_DRILL_BITS; i++){
		if (bit_active[i] == YES){
			
			// Generate the file name for this
			// layer and drill bit size
			sprintf(drill_filename, "%s.#%d.gcode",fname,i);
	
			// Write the file header
			write_drill_header(drill_filename,i);
			// Write the body
			write_drill_body(drill_filename, i);
			// Write the file trailer
			write_drill_trailer(drill_filename);
		}
	}

}

/**
/* set_unique_hole() - Create a list of unique size holes
**/
void set_unique_hole(int hole_size){
	real drill_mm;

	drill_mm = u2mm(hole_size);

	for (int i = 0; i < drill_index; i++){
		if (drill_size_mm[i] == drill_mm)
			return;  // found
	}
	// Not found, include in list
	drill_size_mm[drill_index] = drill_mm;
	drill_index++;
}

/**
/* drill_check() - Check what size of drills has the board
**/
string drill_check(){

	text_buffer = "";
	drill_index = 0;
	drill_size_mm[drill_index] = 0.;

	clean_bits_active();

	if (board) {
		board(B) {
			B.holes(H) set_unique_hole(H.drill);
			B.signals(S) S.vias(V) set_unique_hole(V.drill);
			B.elements(E) {
				E.package.contacts(C) {
					if (C.pad) {
						set_unique_hole(C.pad.drill);
					}
				}
				E.package.holes(H) set_unique_hole(H.drill);
			} 
		}

		// Sort data
		sort(drill_index, index, drill_size_mm);  

		for (int i = 0; i < drill_index; i++){
			sprintf(temp, "%.3f mm\n",drill_size_mm[index[i]]);
			text_buffer = text_buffer + temp;
		}


	}
	return text_buffer;
}

/**
/* drill_main() - main solder paste dispensing code
**/
void drill_main(){

	drill_index = 0;
	clean_bits_active();

	if (board) {
		board(B) {
			B.holes(H) set_hole(H.drill, H.x, H.y);
			B.signals(S) S.vias(V) set_hole(V.drill, V.x, V.y);
			B.elements(E) {
				E.package.contacts(C) {
					if (C.pad) {
						set_hole(C.pad.drill, C.pad.x, C.pad.y);
					}
				}
				E.package.holes(H) set_hole(H.drill, H.x, H.y);
			} 
		}

		// Sort data to optimize coord.
		sort(drill_index, index, drill_size_mm, x, y);  

		// Process layers

		if (GENERATE_TOP_DRILL == YES){
			current_layer = TOP;
			drill_layer(board_name + ".drill.top");
		}

		if (GENERATE_BOTTOM_DRILL == YES){
			current_layer = BOTTOM;
			drill_layer(board_name + ".drill.bot");
		}

	}

}